/* 
 * Author: Cezary Gwizdowski - T-TIZ
 * biblioteka stl, opis algorytmu : sort
 * File: stl_sort.cpp
 */

/*
 * Algorytm sort umoliwia nam wydajne sortowanie dowolnych zbiorw elementw.
 * S dwie wersje algorytmu. Pierwsza pobiera dwa argumenty - sort(first,last), 
 * ktre odpowiednio oznaczaj pierwszy i ostatni element pomidzy
 * ktrymi maj one zosta posortowane. Element first znajduje si z w zbiorze 
 * elementw sortowanych, a element last ju nie - [first,last). Do porwnywania 
 * elementw uywany jest operator "<" , ktrego dziaanie moemy dowolnie 
 * przystosowa dla obiektw sortowanych. Druga wersja algorytmu pobiera 
 * dodatkowo trzeci parametr - sort(first,last,comp). Ostatni parametr oznacza
 * funkcj, ktra zostanie wykorzystana do porwnywanie elementw podczas
 * sortowania. Argumentami tej funkcji s dwa elementy ze zbioru. Zwraca ona 
 * true jeli pierwszy element ma si znale przed drugim i false w przeciwnym 
 * wypadku. Jeli w sortowanym zbiorze bda elementy rwne to nie ma pewnoci 
 * na to e zostan one w oryginalnej kolejnoci.  
 */

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <string>
#include <vector>
#include <sstream>

/* 
 * Algorytm sort biblioteki STL znjaduje si w komponencie algorytm dlatego jeli 
 * chcemy uywa tego algorytmu naley doczy plik nagwkowy <algorytm>. 
 */
#include <algorithm>
/*
 * Biblioteka standardowa (STL) zostaa zawarta w przestrzeni nazw. O tym, e
 * chcemy jej uywa informujemy kompilator w nastpujcy sposb:
 */
using namespace std;

/*
 * Klasa odzwierciedla samochd przechowujc pewne jego parametry.
 */
class Samochod
{
public:
    /* Nazwa samochodu. */
    string nazwa;   
    /* Moc silnika samochodu. */
    int moc ;
    /* Cena samochodu. */
    int cena;

    /* Konstruktor klasy Samochd. */
    Samochod(string nazwa, int moc, int cena) {
        this->nazwa = nazwa;
        this->moc = moc;
        this->cena = cena;
    }
};

/*
 * Funkcja porwnuje moc dwch obiektw typu Samochod. Gdy moc pierwszego jest
 * wiksza funkcja zwraca true. W przeciwnym razie zwraca false. 
 */
bool porownajMocSamochodow(Samochod s1, Samochod s2) { 
    if (s1.moc < s2.moc) return true;
    else return false; 
}

/*
 * Klasa klasyfikator, ktra porwnuje samochody pod wzgldem ceny.
 */
struct KlasyfikatorCenowy {
  bool operator() (Samochod s1, Samochod s2) { 
      if (s1.cena < s2.cena) return true;
      else return false;
  }
} mojKlasyfikatorCenowy; //obiekt klasyfikatora cenowego

int main(int argc, char** argv) {
   
    printf("Przykady uycia algorytmu SORT z biblioteki STL.\n\n");
    
    /*
     * Przykad uycia algorytmu sort z dwoma parametrami na talicy typu int.
     * Do porwnywania elementw uywany jest domylny operator "<" dla int.
     */
    printf("1. Tablica typu int:\n\t");
    int a[8] = {5, 29, 1, -40, -17, 65, 100, -100};
    for (int i=0; i<8; i++) {
        printf("%4d ", a[i]);
    }
    /*
     * Out: 5   29    1  -40  -17   65  100 -100
     */
    printf("\n\nPo uyciu algorytmu sort:\n\t");
    sort(a, a+8); //sortowanie
    for (int i=0; i<8; i++) {
        printf("%4d ", a[i]);
    }
    /* 
     * Out: -100  -40  -17    1    5   29   65  100 
     */

    /*
     * Przykad uycia algorytmu sort z dwoma parametrami na talicy typu int.
     * Sortowane s tylko elementy od 2 do 7.
     * Do porwnywania elementw uywany jest domylny operator "<" dla int.
     */
    printf("\n\n2. Sortowanie ze zmiana zakresu (2-7). Tablica typu int:\n\t");
    int b[8] = {5, 29, 1, -40, -17, 65, 100, -100};
    for (int i=0; i<8; i++) {
        printf("%4d ", b[i]);
    }
    /*
     * Out: 5   29    1  -40  -17   65  100 -100
     */
    printf("\n\nPo uyciu algorytmu sort:\n\t");
    sort(b+2, b+7); //sortowanie
    for (int i=0; i<8; i++) {
        printf("%4d ", b[i]);
    }
    /* 
     * Out: 5   29  -40  -17    1   65  100 -100 
     *  Wita na tym przykadzie, e zostay posortowane jedynie elementy [2,7). 
     */
    
    /*
     * Przykad uycia algorytmu sort z dwoma parametrami na obiekcie typu 
     * vector przechowujcego obiekty typu string. Do porwnywania elementw 
     * uywany jest domylny operator "<" dla string.
     */
    printf("\n\n3. Vector przechowujcy obiekty typu string:\n");
    vector <string> v1;
    v1.push_back("posiedzie");
    v1.push_back("Dobrze");
    v1.push_back("ubrze :)");
    v1.push_back("jest");
    v1.push_back("przy");
    for(int i = 0;i < v1.size();i++){
        cout << "\t" << i << ":" << v1.at(i) << "\n";
    }
    /*
     *	Out: 0:posiedzie
     *       1:Dobrze
     *       2:ubrze :)
     *       3:jest
     *       4:przy
     */
    sort(v1.begin(), v1.end()); //sortowanie
    printf("\nPo uyciu algorytmu sort:\n");
    for(int i = 0;i < v1.size();i++){
        cout << "\t" << i << ":" << v1.at(i) << "\n";
    }
    /* 
     * Out: 0:Dobrze
     *      1:jest
     *      2:posiedzie
     *      3:przy
     *      4:ubrze :) 
     * W tym wypadku elementy zostay posortowane alfabetycznie.
     */
    
    /*
     * Przykad uycia algorytmu sort na obiektach typu Samochod. 
     * Do porwnania tych obiektw zostanie wykorzystana funkcja 
     * porownajMocSamochodow() porwnujca moc dwch samochodw.
     */
    printf("\n4. Vector przechowujcy obiekty typu Samochod:\n");
    vector <Samochod> v2;
    v2.push_back(Samochod("Lamborghini Gallardo", 512, 723450));
    v2.push_back(Samochod("Porsche 911", 190, 346000));
    v2.push_back(Samochod("Ferrari 250 S", 210, 1300000));
    for(int i = 0;i < v2.size();i++){
        cout << "\t[Samochd] Nazwa: " << v2.at(i).nazwa << " , Cena: " 
            << v2.at(i).cena << " , Moc: " << v2.at(i).moc << "\n"; 
    }
    sort(v2.begin(), v2.end(), porownajMocSamochodow); //sortowanie
    printf("\nPo uyciu algorytmu sort - porwnanie za pomoc funkcji porwnujcej moc samochodw:\n");
    for(int i = 0;i < v2.size();i++){
        cout << "\t[Samochd] Nazwa: " << v2.at(i).nazwa << " , Cena: " 
            << v2.at(i).cena << " , Moc: " << v2.at(i).moc << "\n"; 
    }
    /* 
     * Out: [Samochd] Nazwa: Porsche 911 , Cena: 346000 , Moc: 190
     *      [Samochd] Nazwa: Ferrari 250 S , Cena: 1300000 , Moc: 210
     *      [Samochd] Nazwa: Lamborghini Gallardo , Cena: 723450 , Moc: 512
     */
    
    /*
     * Przykad uycia obiektu do porwnywania elementw podczas sortowania.
     */
    sort(v2.begin(), v2.end(), mojKlasyfikatorCenowy); //sortowanie
    printf("\nPo uyciu algorytmu sort - porwnanie za pomoc obiektu klasyfikatora cenowego porwnujcego cene samochodw:\n");
    for(int i = 0;i < v2.size();i++){
        cout << "\t[Samochd] Nazwa: " << v2.at(i).nazwa << " , Cena: " 
            << v2.at(i).cena << " , Moc: " << v2.at(i).moc << "\n"; 
    }
    /*
     * Out: [Samochd] Nazwa: Porsche 911 , Cena: 346000 , Moc: 190
     *      [Samochd] Nazwa: Lamborghini Gallardo , Cena: 723450 , Moc: 512
     *      [Samochd] Nazwa: Ferrari 250 S , Cena: 1300000 , Moc: 210
     */
    
    return (EXIT_SUCCESS);
}
